# Chapter 2. Variables and Basic Types

## 2.3 Compound Types

A **Compound type** is a data type that's defined using another data type. (e.g. **reference** & **pointers**)

* A declaration of data: **base type** followed by a list of **declarators**.
* Simple declaration: declarator are just variable name

### 2.3.1 References

Note: After reading the book, I use [Reference declaration from cppreference.com](https://en.cppreference.com/w/cpp/language/reference) for better explanation

Reference declaration declares a named variable as a **reference**, that is, an alias to an already-existing object or function.

2 kinds of reference variable declaration:

* Lvalue reference
* Rvalue reference (check chapter 13.6.1)

**reference** (usually means "lvalue reference") declaration syntax: `S& D;`, declares declarator D as an `lvalue reference` to type determined by S

```cpp
int ival = 1024;
int &refVal = ival;  // refVal refers to (is another name for) ival
int &refVal2;        // error: a reference must be initialized
```

* When decleared/initizlied, the reference is bound to the initialized object
* After declaration, all operation on reference are operated on bounded obj

```cpp
refVal = 2 // assigns 2 to the object to which refVal refers, i.e., to ival
int ii = refVal; // same as ii = ival
```

### 2.3.2 Pointers

A **pointer** is a compound type "points to" another data type:

* Like reference: pointers are used for indirect access
* Unlike reference: pointer itself is an object. Hence, it can be reassigned/copied. No need to be initialized when declared

Pointer declaration: `S* D`, where declares declarator D as a pointer to the type determined by S; `*` needed for each pointer variable

```cpp
int *ip1, *ip2;  // both ip1 and ip2 are pointers to int
double dp, *dp2; // dp2 is a pointer to double; dp is a double
```

#### Taking the address of an Object

Pointer hold address of another obj. Address are obtained using `&`

```cpp
int ival = 42;
int *p = &ival; // p holds the address of ival; p is a pointer to ival
```

* Reference are not obj => they don't have address.
* Type of pointer & obj must match

```cpp
double dval;
double *pd = &dval;  // ok: initializer is the address of a double
double *pd2 = pd;    // ok: initializer is a pointer to double
int *pi = pd;  // error: types of pi and pd differ
pi = &dval;    // error: assigning the address of a double to a pointer to int
```

#### Pointer Value (states)

Pointer value can be in 4 states:

1. Point to an obj
2. Point to the location just immediately past the end of an obj
3. Point to null; i.e. bound to none
4. Invalid; values other than preceding 3; these pointers generate error

#### Using a Pointer to Access Obj

**Dereference operator** (`*` operator):

```cpp
int ival = 42;
int *p = &ival; // p holds the address of ival; p is a pointer to ival
cout << *p;     // * yields the object to which p points; prints 42
```

#### Null Pointers

* **Null Pointer** points to no one.
* Developer can check whether a pointer pointing to NULL.

Multiple values to be assigned as null:

* `nullptr`: most directed, introduced in C++11
* `0`
* `NULL`: a **preprocessor variable** used by older programes, defined in `cstdlib` header. Replaced by preprocessor before compilation.

```cpp
int *p1 = nullptr; // equivalent to int *p1 = 0;
int *p2 = 0;       // directly initializes p2 from the literal constant 0
// must #include cstdlib
int *p3 = NULL;    // equivalent to int *p3 = 0;
```

* **Advice: Initialize all Pointers**. Uninitialized pointers are common source of run-time errors.
* Difference btw reference & pointer:
  * reference is not obj; must be initalized; cannot be reassigned

#### `void*` Pointers

`void*` is a special pointer type that can hold address of any object. Type of the object at that address is unknown:

```cpp
double obj = 3.14, *pd = &obj;
// ok: void* can hold the address value of any data pointer type
void *pv = &obj;  // obj can be an object of any type
pv = pd;          // pv can hold a pointer to any type
```

Utility of `void*` pointer:

* Compare it to another pointer
* Pass it to or return it from a function
* Assign it to another `void*` pointer

Details are covered in Chapter 19.1.1

### 2.3.3 Understanding Compound Type Declarations

A variable definition consists of a base type and a list of declarators. Each declarator can relate its variable to the base type differently from others. e.g. shown below

```cpp
// i is an int; p is a pointer to int; r is a reference to int
int i = 1024, *p = &i, &r = i;
```

#### Defining Multiple Variables

It's easy to be confused and think type modifier applies to all variables

```cpp
int* p;  // legal but might be misleading
```

```cpp
int* p1, p2; // p1 is a pointer to int; p2 is an int
```

#### Pointers to Pointers

A pointer is an object in memory, so like any object it also has an address.

* `**` for a pointer to a pointer
* `***` for a pointer to a pointer to a pointer

```cpp
int ival = 1024;
int *pi = &ival;    // pi points to an int
int **ppi = &pi;    // ppi points to a pointer to an int
```

Dereferencing also will be cascaded

```cpp
cout << "The value of ival\n"
     << "direct value: " << ival << "\n"
     << "indirect value: " << *pi << "\n"
     << "doubly indirect value: " << **ppi
     << endl;
```

#### References to Pointers

As a pointer is an object, we can define a reference to a pointer:

```cpp
int i = 42;
int *p;        // p is a pointer to int
int *&r = p;   // r is a reference to the pointer p
r = &i; //  r refers to a pointer; assigning &i to r makes p point to i
*r = 0; //  dereferencing r yields i, the object to which p points; changes i to 0
```

Way to understand: read `r` right to left; `*&r` means the r is a reference that refer to a pointer.

## 2.4 CONST Qualifier

* Defining the variable's type as **const**
* const object must be initialized when creation.

```cpp
const int i = get_size();  // ok: initialized at run time
const int j = 42;          // ok: initialized at compile time
const int k;               // error: k is uninitialized const
```

#### Initialization** and `const`

* `const` types has restriction in terms of operations it allows

```cpp
int i = 42;
const int ci = i;    // ok: the value in i is copied into ci
int j = ci;          // ok: the value in ci is copied into j
```

#### By Default, `const` Objects Are Local to a File

* Compiler will replace const objects during compilation
* Hence, compiler need to know variable's initializer during compilation (not run time)
* Hence, variable must be defined in every file that wants to use the value
* So, normally, same name  with `const` in multiple files are separate.

Q: How to share `const` across multiple files? i.e. Define the `const` in 1 file, and declare it in other files to use it?

A: add `extern` on both definition & declaration

```cpp
// file_1.cc defines and initializes a const that is accessible to other files
extern const int bufSize = fcn();
// file_1.h
extern const int bufSize; // same bufSize as defined in file_1.cc
```

### 2.4.1 Reference to `const`

Q: How to bind a reference to an object of a `const` type.

A: use **reference to const**: a reference to a `const` type. As it's bound to a const, it cannot be used to change value

Syntax of reference to const: `const int &D`

```cpp
const int ci = 1024;
const int &r1 = ci;   // ok: both reference and underlying object are const
r1 = 42;              // error: r1 is a reference to const
int &r2 = ci;         // error: non const reference to a const object
```

#### Initialization and Reference to `const`

Exception to be noted:

1. We can bind a **reference to const** to a **nonconst object**: When we bind a reference to an object of different type, compiler will transform the RHS to a proper data type to fit LHS (e.g. create a temporary `const int temp` that equal to float value, then reference to that temporary value)
   1. Normally don't do that

```cpp
double dval = 3.14;
const int &ri = dval;


const int &ri = dval;
//equal to
const int temp = dval;
const int &ri = temp
```

#### A Reference to `const` May Refer to an Obj that's not `const`

Binding a **reference to const** to an object only restrict the operation we can do on reference, not the underlying obj itself.

```cpp
int i = 42;
int &r1 = i;          // r1 bound to i
const int &r2 = i;    // r2 also bound to i; but cannot be used to change i
r1 = 0;               // r1 is not const; i is now 0
r2 = 0;               // error: r2 is a reference to const
```

### 2.4.2 Pointers and `const`

* **Pointer to const** cannot be used to change the object. 
* Only can store the address of a `const` object in a **pointer to `const` (i.e. const pointer)**

Syntax: `const S *D = &`

```cpp
const double pi = 3.14;   // pi is const; its value may not be changed
double *ptr = &pi;        // error: ptr is a plain pointer
const double *cptr = &pi; // ok: cptr may point to a double that is const
*cptr = 42;               // error: cannot assign to *cptr
```

Notes:

1. we can store a nonconst object's address in a **pointer to const**

It's helpful to think of **pointers** & **references** to `const` as pointers or references "that think they point or refer to `const`."

#### `const` Pointers

A pointer can also be `const`. Syntax: `const double *const pip`

???

### 2.4.3 Top-Level `const`

* **top-level const**: pointer itself is `const`
* **low-level const**: pointer point to a `const` object

???

### 2.4.4 `constexpr` and Constant Expressions

* **Constant expression** = an expression whose value cannot change & can be evaluated at compile time.
* e.g. 
  * literal
  * `const` object initialized from constant expression

```cpp
const int max_files = 20;    // max_files is a constant expression
const int limit = max_files + 1; // limit is a constant expression
int staff_size = 27;       // staff_size is not a constant expression
const int sz = get_size(); // sz is not a constant expression
```

#### `constexpr` Variables (C++11)

In large system, whether an initializer is a constant expression is hard to determine. Hence, we can ask compiler to verify whether a variable is constant expression using `constexpr` declaration.

```cpp
constexpr int mf = 20;        // 20 is a constant expression
constexpr int limit = mf + 1; // mf + 1 is a constant expression
constexpr int sz = size();    // ok only if size is a constexpr function
```

After C++11, It's best practice to use `constexpr` instead of `const`

#### Literal Types

**Literal Types** = types we can use in a `constexpr`

Types known are literal types:

* arithmetic
* reference
* pointer types
* `nullptr` or `0` as constant expression
* Address of an obj defined outside function (check 6.1.1)

Not literal:

* variable ordinarily defined inside function (check 6.1.1)

#### Pointers and `constexpr`

When define a pointer in a `constexpr` declaration, it applies to pointer not underlying obj

```cpp
const int *p = nullptr;     // p is a pointer to a const int
constexpr int *q = nullptr; // q is a const pointer to int
```

a `constexpr` pointer maynot point to a `const` type:

```cpp
constexpr int *np = nullptr; // np is a constant pointer to int that is null
int j = 0;
constexpr int i = 42;  // type of i is const int
// i and j must be defined outside any function
constexpr const int *p = &i; // p is a constant pointer to the const int i
constexpr int *p1 = &j;      // p1 is a constant pointer to the int j
```

## 2.5 Dealing With Types

Complicated program results to complicated types:

1. Some types are tedious/lengthy form
2. Hard to determine the exact type we need.

### 2.5.1 Type Aliases

**Type Alias** = a name that's a synoym for another type. So we can simplify complicated type name.

Two way of definition:

1. `typedef`: traditional
2. **alias declaration**: since c++11

Syntax of **typedef**:

```cpp
typedef double wages;   // wages is a synonym for double
typedef wages base, *p; // base is a synonym for double, p for double*
```

* declarator can include type modifiers (`&` `*`) to form compound type

Syntax for **alias declaration**:

```cpp
using SI = Sales_item;  // SI is a synonym for Sales_item
```

After definition of type alias, it is used as a type name and can appear wherever a type name is:

```cpp
wages hourly, weekly;    // same as double hourly, weekly;
SI item;                 // same as Sales_item item
```

#### Pointers, `const` and Type Aliases

Mixed type modifier and type aliases together

```cpp
typedef char *pstring; // pstring is an alis for type char*
const pstring cstr = 0; // cstr is a constant pointer to char
const pstring *ps;      // ps is a pointer to a constant pointer to char
```

* type of `pstring` is "pointer to char"
* type of `const pstring` is a "constant pointer to char", not a pointer to `const char`

???

### 2.5.2 The `auto` Type Specifier (C++11)

* Sometimes, it's impossible to determine the type of an expression when writing it.
* `auto` type specifier from C++11 tell compiler to **deduce the type from initializer**.
* variable use `auto` must have an initializer

```cpp
// the type of item is deduced from the type of the result of adding val1 and val2
auto item = val1 + val2; // item initialized to the result of val1 + val2
```

Multiple variable declaration using `auto`:

```cpp
auto i = 0, *p = &i;      // ok: i is int and p is a pointer to int
auto sz = 0, pi = 3.14;   // error: inconsistent types for sz and pi
```

#### Compound Types, `const` and `auto`

* When use reference as initializer for `auto`, compiler use underlying object's type for `auto`'s type deduction

```cpp
int i = 0, &r = i;
auto a = r;  // a is an int (r is an alias for i, which has type int)
```

* `auto` normally ignores top-level const, while low-level const (e.g. an initializer is a pointer to const) are kept

```cpp
const int ci = i, &cr = ci; // ci is constant integer, cr is a reference to constant integer (low-level)
auto b = ci;  // b is an int (top-level const in ci is dropped)
auto c = cr;  // c is an int (cr is an alias for ci whose const is top-level)
auto d = &i;  // d is an int*(& of an int object is int*)
auto e = &ci; // e is const int*(& of a const object is low-level const)
```

??? Concepts are confusing ???

### 2.5.3 The `decltype` Type Specifier

Type specifier **decltype** returns type of its operand (i.e. compiler deduces from expression but don't use the expression to initialize variable)

```cpp
decltype(f()) sum = x; // sum has whatever type f returns
```

Explain:

* compiler doesn't call `f`, but uses the type that such a call would return as type for `sum`

How `decltype` handles **top-level const** and **reference**: returns type of that variable, including top-level const and references:

```cpp
const int ci = 0, &cj = ci;
decltype(ci) x = 0; // x has type const int
decltype(cj) y = x; // y has type const int& and is bound to x
decltype(cj) z;     // error: z is a reference and must be initialized
```

#### `decltype` and References

`decltype` is the only context in which a variable defined as a reference is not treated as synonym for the referred object

`decltype` returns a reference type for expression that yield objects that can stand on LHS of assignment

```cpp
// decltype of an expression can be a reference type
int i = 42, *p = &i, &r = i;
decltype(r + 0) b;  // ok: addition yields an int; b is an (uninitialized) int
decltype(*p) c;     // error: c is int& and must be initialized
```

* `decltype(r)` returns a reference type
* If we want the type of referred object, use `r` in an expression (e.g. `r+0`)
* `decltype(*p)` returns a reference, must be initialized ????

??? Not Finished ???